"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.render = exports.renderServer = exports.checkModules = exports.readPageHtml = void 0;

var _regenerator = require("@babel/runtime/regenerator");

var _fs = require("fs");

var _vueServerRenderer = require("vue-server-renderer");

var _common = _interopRequireWildcard(require("@srejs/common"), true);

var _serializeJavascript = require("serialize-javascript");

var _webpack = require("@srejs/webpack");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * 写入文件,存在则覆盖
 * @param {*} path 文件名称
 * @param {*} Content 内容
 */
var writeFile = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/_regenerator.mark(function _callee(path, Content) {
    return _regenerator.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            return _context.abrupt("return", new Promise(function (resolve) {
              _fs.writeFile(path, Content, {
                encoding: 'utf8'
              }, function (err) {
                if (err) {
                  resolve(false);
                } else {
                  resolve(true);

                  _common.Logger.info("SSR:Page component ".concat(path, " successfully writes the server rendering cache"));
                }
              });
            }));

          case 1:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function writeFile(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * 用Promise封装异步读取文件方法
 * @param  {string} page html文件名称
 * @return {promise}
 */


var readPageHtml = function readPageHtml(page) {
  return new Promise( /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regenerator.mark(function _callee2(resolve, reject) {
      var viewUrl, readFileSync, htmlString;
      return _regenerator.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              viewUrl = "".concat(_common.clientDir, "/").concat(page, "/").concat(page, ".html");

              readFileSync = function readFileSync() {
                _fs.readFile(viewUrl, 'utf8', function (err, htmlString) {
                  if (err) {
                    reject(err);

                    _common.Logger.error('SSR: The client code is being compiled, please refresh the browser later...');
                  } else {
                    resolve(htmlString);
                  }
                });
              };

              if (_common["default"].isDev()) {
                try {
                  htmlString = _webpack.VueDevMiddlewareFileSystem.readFileSync(viewUrl, 'utf-8');
                  resolve(htmlString);
                } catch (error) {
                  readFileSync();
                }
              } else {
                readFileSync();
              }

            case 3:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function (_x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }());
};

exports.readPageHtml = readPageHtml;

var writeFileHander = function writeFileHander(cacheDir, cacheUrl, Content) {
  _fs.exists(cacheUrl, function (exists) {
    if (exists) {
      writeFile(cacheUrl, Content);
    } else {
      _fs.mkdir(cacheDir, {
        recursive: true
      }, function (err) {
        if (err) {
          _common.Logger.error("SSR:".concat(err.stack));
        } else {
          writeFile(cacheUrl, Content);
        }
      });
    }
  });
};

var checkModules = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/_regenerator.mark(function _callee3(page) {
    var jspath, jsClientdir, compiler, _compiler;

    return _regenerator.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            jspath = "".concat(_common.serverDir, "/").concat(page, "/").concat(page, ".js");
            jsClientdir = "".concat(_common.clientDir, "/").concat(page);

            if (_fs.existsSync(jspath)) {
              _context3.next = 6;
              break;
            }

            //服务端代码打包
            compiler = new _webpack.WebpackVue(page, _common["default"].isDev(), true);
            _context3.next = 6;
            return compiler.run();

          case 6:
            if (!(!_fs.existsSync(jsClientdir) && !_common["default"].isDev())) {
              _context3.next = 10;
              break;
            }

            //客户端代码打包
            _compiler = new _webpack.WebpackVue(page, _common["default"].isDev());
            _context3.next = 10;
            return _compiler.run();

          case 10:
            return _context3.abrupt("return", jspath);

          case 11:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function checkModules(_x5) {
    return _ref3.apply(this, arguments);
  };
}();
/**
 * Router类型页面渲染解析
 * @param {*} ctx
 * @param {*} next
 */


exports.checkModules = checkModules;

var renderServer = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/_regenerator.mark(function _callee4(ctx) {
    var initProps,
        ssr,
        context,
        _ctx$SSRKEY,
        page,
        query,
        createApp,
        jspath,
        Html,
        App,
        document,
        _args4 = arguments;

    return _regenerator.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            initProps = _args4.length > 1 && _args4[1] !== undefined ? _args4[1] : {};
            ssr = _args4.length > 2 && _args4[2] !== undefined ? _args4[2] : true;
            context = {
              url: ctx.req.url
            };
            _ctx$SSRKEY = ctx[_common.SSRKEY], page = _ctx$SSRKEY.page, query = _ctx$SSRKEY.query;
            query = (0, _common.filterXssByJson)(query);

            if ((0, _webpack.getVueEntryList)().has(page)) {
              _context4.next = 7;
              break;
            }

            return _context4.abrupt("return", "Page component ".concat(page, " does not exist, please check the pages folder"));

          case 7:
            _context4.next = 9;
            return checkModules(page);

          case 9:
            jspath = _context4.sent;

            try {
              // eslint-disable-next-line no-undef
              if (_common["default"].isDev()) {
                delete require.cache[require.resolve(jspath)];
              }

              createApp = require(jspath);
              createApp = createApp["default"] ? createApp["default"] : createApp;
            } catch (error) {
              // eslint-disable-next-line no-console
              _common.Logger.error("SSR: ".concat(page, "Please check whether there are APIs in the code that the server does not support when rendering,\n             such as window, locaction, navigator, etc"));

              _common.Logger.error(error.stack);
            }

            if (_typeof(initProps) === 'object') {
              context = Object.assign({
                title: '',
                meta: ''
              }, context, initProps);
            }

            context.ssrData = {
              page: page,
              path: ctx[_common.SSRKEY].path,
              query: query,
              options: ctx[_common.SSRKEY].options
            };
            Html = '';

            if (!ssr) {
              _context4.next = 33;
              break;
            }

            _context4.prev = 15;
            _context4.next = 18;
            return createApp(context);

          case 18:
            App = _context4.sent;
            _context4.next = 21;
            return renderTostring(App, context, page);

          case 21:
            Html = _context4.sent;
            _context4.next = 31;
            break;

          case 24:
            _context4.prev = 24;
            _context4.t0 = _context4["catch"](15);
            ctx[_common.SSRKEY].options.ssr = false;

            _common.Logger.warn('SSR:服务端渲染异常，降级使用客户端渲染！' + JSON.stringify(_context4.t0));

            _context4.next = 30;
            return readPageString(page, context);

          case 30:
            Html = _context4.sent;

          case 31:
            _context4.next = 36;
            break;

          case 33:
            _context4.next = 35;
            return readPageString(page, context);

          case 35:
            Html = _context4.sent;

          case 36:
            document = renderDocumentHead(Html, context);
            return _context4.abrupt("return", document);

          case 38:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, null, [[15, 24]]);
  }));

  return function renderServer(_x6) {
    return _ref4.apply(this, arguments);
  };
}();

exports.renderServer = renderServer;

var readPageString = /*#__PURE__*/function () {
  var _ref5 = _asyncToGenerator( /*#__PURE__*/_regenerator.mark(function _callee5(page, context) {
    var temp;
    return _regenerator.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return readPageHtml(page);

          case 2:
            temp = _context5.sent;
            return _context5.abrupt("return", injectScriptInitProps(temp, context));

          case 4:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));

  return function readPageString(_x7, _x8) {
    return _ref5.apply(this, arguments);
  };
}();

var renderTostring = /*#__PURE__*/function () {
  var _ref6 = _asyncToGenerator( /*#__PURE__*/_regenerator.mark(function _callee6(App, context, page) {
    var template, renderer;
    return _regenerator.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return readPageString(page, context);

          case 2:
            template = _context6.sent;
            renderer = (0, _vueServerRenderer.createRenderer)({
              template: template
            });
            return _context6.abrupt("return", new Promise(function (resove, reject) {
              renderer.renderToString(App, context, function (err, html) {
                if (err) {
                  reject(err);
                  return;
                }

                resove(html);
              });
            }));

          case 5:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  }));

  return function renderTostring(_x9, _x10, _x11) {
    return _ref6.apply(this, arguments);
  };
}();

var injectScriptInitProps = function injectScriptInitProps(temp, context) {
  var contents = temp.split('<!--vue-ssr-outlet-->');

  if (contents.length == 1) {
    console.error('SSR:警告！自定义html文件中请在body下追加<!--vue-ssr-outlet-->占位符！https://ssr.vuejs.org/zh/guide/#%E4%BD%BF%E7%94%A8%E4%B8%80%E4%B8%AA%E9%A1%B5%E9%9D%A2%E6%A8%A1%E6%9D%BF');
  }

  var data = Object.assign({}, context);
  var ssrData = data.ssrData;
  delete data.state;
  delete data.ssrData;
  delete context.ssrData;
  ssrData.initProps = data;
  return contents[0] + '<!--vue-ssr-outlet-->' + '<script>window.__SSR_DATA__=' + _serializeJavascript(ssrData, {
    isJSON: true
  }) + '</script>' + contents[1];
};
/**
 * 获取服务端渲染直出资源
 * @param {*} ctx
 */


var render = /*#__PURE__*/function () {
  var _ref7 = _asyncToGenerator( /*#__PURE__*/_regenerator.mark(function _callee8(ctx, initProps) {
    var page, _ctx$SSRKEY$options, cache, ssr;

    return _regenerator.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            page = ctx[_common.SSRKEY].page;
            _ctx$SSRKEY$options = ctx[_common.SSRKEY].options, cache = _ctx$SSRKEY$options.cache, ssr = _ctx$SSRKEY$options.ssr;
            return _context8.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref8 = _asyncToGenerator( /*#__PURE__*/_regenerator.mark(function _callee7(resolve) {
                var ssrCacheDir, ssrCacheUrl, rs, document;
                return _regenerator.wrap(function _callee7$(_context7) {
                  while (1) {
                    switch (_context7.prev = _context7.next) {
                      case 0:
                        if (ssr) {
                          _context7.next = 6;
                          break;
                        }

                        _context7.t0 = resolve;
                        _context7.next = 4;
                        return renderServer(ctx, initProps, false);

                      case 4:
                        _context7.t1 = _context7.sent;
                        return _context7.abrupt("return", (0, _context7.t0)(_context7.t1));

                      case 6:
                        ssrCacheDir = "".concat(_common.cacheDir).concat(ctx.url, "/");
                        ssrCacheUrl = "".concat(_common.cacheDir).concat(ctx.url, "/").concat(page, ".html");

                        if (!(_common["default"].isDev() || !cache)) {
                          _context7.next = 16;
                          break;
                        }

                        _context7.t2 = resolve;
                        _context7.next = 12;
                        return renderServer(ctx, initProps, true);

                      case 12:
                        _context7.t3 = _context7.sent;
                        return _context7.abrupt("return", (0, _context7.t2)(_context7.t3));

                      case 16:
                        if (!_fs.existsSync(ssrCacheUrl)) {
                          _context7.next = 21;
                          break;
                        }

                        // ssr缓存模式，执行一次ssr 第二次直接返回缓存后的html静态资源
                        rs = _fs.createReadStream(ssrCacheUrl, 'utf-8');
                        return _context7.abrupt("return", resolve(rs));

                      case 21:
                        _context7.next = 23;
                        return renderServer(ctx, initProps, true);

                      case 23:
                        document = _context7.sent;
                        process.nextTick(function () {
                          writeFileHander(ssrCacheDir, ssrCacheUrl, document); //异步写入服务器缓存目录
                        });
                        return _context7.abrupt("return", resolve(document));

                      case 26:
                      case "end":
                        return _context7.stop();
                    }
                  }
                }, _callee7);
              }));

              return function (_x14) {
                return _ref8.apply(this, arguments);
              };
            }()));

          case 3:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8);
  }));

  return function render(_x12, _x13) {
    return _ref7.apply(this, arguments);
  };
}();
/**
 * 从initprops中读取title,keyworkds desription写入到html中
 * @param {*} html
 * @param {*} initProps
 */


exports.render = render;

var renderDocumentHead = function renderDocumentHead(document, initProps) {
  var resStr = '';
  ['title', 'keywords', 'description'].forEach(function (key) {
    if (initProps[key]) {
      if (key == 'title') {
        resStr = "<title>".concat(initProps[key], "</title>");
        document = document.replace(/<title>.*<\/title>/, resStr);
      } else {
        var reg = new RegExp("<meta\\s+name=[\\\"']".concat(key, "[\\\"'].*?content=[\\\"']([\\S\\s]*?)[\\\"'].*?[\\/]?>"));
        resStr = "<meta name=\"".concat(key, "\" content=\"").concat(initProps[key], "\">");

        if (document.match(reg)) {
          //replace
          document = document.replace(reg, resStr);
        } else {
          //add
          document = document.replace('<title>', "<meta name=\"".concat(key, "\" content=\"").concat(initProps[key], "\">\r\n    <title>"));
        }
      }
    }
  });
  return document;
};